#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
python URL处理的工具集
'''

from urllib.parse import urlparse


def get_domain_from_url(url):
    resp = urlparse(url)
    return resp.scheme + '://' + resp.netloc





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import subprocess


def run_bash(cmd, return_result=False):
    """
    运行shell命令
    """
    p = subprocess.Popen(cmd, shell=True, 
                            stdout=subprocess.PIPE, 
                            stderr=subprocess.PIPE)
    error = p.stderr.read().decode('utf-8')
    if error:
        raise Exception(error)
    if return_result:
        result = p.stdout.read().decode('utf-8')
        return result




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
python 文件处理的工具集
'''

import os
import hashlib
from iotcaspy import ShellUtils
from pathlib2 import Path


def file_to_list(filepath):
    """
    读取txt文件的每一列并保存为list
    :param filepath: 
    :return: list
    """
    with open(filepath, 'r') as f:
        result = [line.strip() for line in f if line]# line后面会带有 '\n'，需要加strip()方法
    return result


def get_all_filenames(dirpath):
    """
    获取目录下所有文件名
    """
    all_filenames = list()
    for root, dirs, files in os.walk(dirpath):
        if files:
            for file in files:
                all_filenames.append(os.path.join(root, file))
    return all_filenames


def get_file_md5(fname):
    """
    获得文件的MD5值
    """
    m = hashlib.md5()
    with open(fname, 'rb') as fobj:
        while True:
            data = fobj.read(4096)
            if not data:
                break
            m.update(data)
    return m.hexdigest().upper()


def get_file_sha256(fname):
    """
    获得文件的SHA256值
    """
    m = hashlib.sha256()
    with open(fname, 'rb') as fobj:
        while True:
            data = fobj.read(4096)
            if not data:
                break
            m.update(data)
    return m.hexdigest().upper()


def get_bytesize_from_str(size_str):
    """
    将字符串表示的文件大小转换成以B为单位的数值
    输入示例：3.45KB
    """
    size_str = size_str.upper()
    unit_mapping = {
        'TB': 1024**4, 
        'GB': 1024**3, 
        'MB': 1024**2, 
        'KB': 1024**1, 
        'B': 1, 
    }
    for unit in unit_mapping:
        if unit in size_str:
            val = float(size_str.rsplit(unit, 1)[0].strip())
            val = int(val*unit_mapping[unit])
            break
    return val


def save_strlist_to_file(filepath, strlist):
    """
    将字符串序列写入到文件中
    """
    with open(filepath, 'w') as f:
        for line in strlist:
            f.write(line+'\n')


import pickle
def save_obj_to_file(obj, filepath):
    """
    将Python对象保存到文件中
    """
    if not filepath.endswith('.pkl'):
        filepath += '.pkl'
    with open(filepath, 'wb') as f:
        pickle.dump(obj, f)


def load_obj_from_file(filepath):
    """
    从文件中载入Python对象
    """
    assert os.path.exists(filepath)
    with open(filepath, 'rb') as f:
        obj = pickle.load(f)
    return obj


def get_file_line_cnts(filepath):
    """
    获得文件行数
    """
    cnts = 0
    with open(filepath, 'r') as f:
        for _ in f:
            cnts += 1
    return cnts


def random_split_file(filepath, percent, new_filepath_list=None):
    """
    随机按比例拆分文件
    """
    if not isinstance(filepath, Path):
        filepath = Path(filepath)
    if new_filepath_list is None:
        new_filepath_list = [filepath.parent/(filepath.stem+'.split1'+filepath.suffix), 
            filepath.parent/(filepath.stem+'.split2'+filepath.suffix), ]
    assert len(new_filepath_list) == 2
    filesize = get_file_line_cnts(filepath)
    subfilepath1, subfilepath2 = str(new_filepath_list[0]), str(new_filepath_list[1])
    file1_size = int(filesize*percent)
    file2_size = filesize - file1_size
    # 生成文件：
    tmp_filepath = str(filepath.parent/(filepath.name+'.tmp'))
    ShellUtils.run_bash('shuf {} -o {}'.format(filepath, tmp_filepath))
    ShellUtils.run_bash('head -n {} {} > {}'.format(file1_size, tmp_filepath, subfilepath1))
    ShellUtils.run_bash('head -n {} {} > {}'.format(file2_size, tmp_filepath, subfilepath2))
    ShellUtils.run_bash('rm {}'.format(tmp_filepath))
    return (subfilepath1, file1_size), (subfilepath2, file2_size)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd

'''
Pandas 工具类
'''


def df_change_type(df, origin_type, target_type):
    """
    将DataFrame中类型为origin_type的列全部转化为target_type。大数据量的情况下非常耗时，不建议使用。

    :param df: 不带NaN的DataFrame
    :param origin_type: list类型的dtype
    :param target_type: 目标dtype
    :return: DataFrame
    """
    target_columns = list()
    for column in df.columns:
        if df[column].dtype in origin_type:
            target_columns.append(column)
    if target_columns is not None:
        df[target_columns] = df[target_columns].astype(target_type)
    return df










#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
if sys.version_info < (3, 0):
    sys.exit('Python 3.0 or greater is required.')


try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup
    
    def find_packages(where='.'):
        # os.walk -> list[(dirname, list[subdirs], list[files])]
        return [folder.replace("/", ".").lstrip(".")
                for (folder, _, fils) in os.walk(where)
                if "__init__.py" in fils]

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name='iotcaspy',                                    # 项目名称
    version="0.1.0",                                    # 项目版本号
    description=(                                       # 项目的简单描述
        'A package from IoTCAS'
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Superin',                                   # 作者名
    author_email='1210065650@qq.com',                   # 作者邮箱
    maintainer='Superin',                               # 维护人员名称
    maintainer_email='felix2@foxmail.com',              # 维护人员邮箱
    license='MIT License',
    packages=find_packages(),
    platforms=["ubuntu", "windows"],                    # 也可以是 ["all"]
    url='https://github.com/Yehrin/iotcaspy',           # Github 链接
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'sqlalchemy',
        'python-dateutil',
        'python-levenshtein',
        'requests',
    ]
)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from concurrent import futures
import os
import datetime
import argparse
from functools import partial
from retry import retry
from ShellUtils import run_bash


def multiproc_handle(func, paralist, max_workers=4):
    """
    多进程处理，返回一个迭代器
    """
    executor = futures.ProcessPoolExecutor(max_workers=max_workers)
    results = executor.map(func, paralist)
    return results


def download_single_file(filepath, save_dir='./'):
    """
    下载单个文件
    返回：state, path
    """
    save_filepath = os.path.join(save_dir, filepath.rsplit('/', 1)[-1])
    if os.path.exists(save_filepath):
        return True, save_filepath
    if not os.path.exists(save_dir):
        os.makedirs(save_dir)
    cmd = 'aria2c -x4 -d %s %s' % (save_dir, filepath)
    try:
        run_bash(cmd)
    except:
        pass
    # 判断文件是否生成
    if os.path.exists(save_filepath):
        return True, save_filepath
    else:
        return False, filepath


def multiproc_download(urls_filepath, save_dir='./', max_workers=4):
    """
    读取给定的URLs文件，并多进程下载
    返回的是一个生成器Generator
    """
    if not os.path.exists(urls_filepath):
        raise Exception('URL文件不存在！')
    if not os.path.exists(save_dir):
        os.makedirs(save_dir)
    with open(urls_filepath, 'r') as f:
        download_urls = [url.strip() for url in f.readlines()]
    download_states = multiproc_handle(partial(download_single_file, save_dir=save_dir), download_urls, max_workers=max_workers)
    return download_states


def display_download_state(states, failed_filepath=None):
    """
    显示下载状态并返回下载失败的URL
    """
    failed_urls = []
    successful_cnt = failed_cnt = 0
    for state, path in states:
        if state:
            # 下载成功
            successful_cnt += 1
            pass
        else:
            failed_cnt += 1
            failed_urls.append(path)
        date = datetime.datetime.now()
        date_str = date.strftime('%Y-%m-%d %H:%M:%S')
        print('\r截止%s，总共下载%d个文件，其中：成功 %d，失败 %d。' % (date_str, failed_cnt+successful_cnt, successful_cnt, failed_cnt), end="")
    print('')
    if len(failed_urls) and failed_filepath:
        with open(failed_filepath, 'w') as f:
            for url in failed_urls:
                f.write(url+'\n')
    
    return failed_urls


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='批量下载URL文件中的文件')
    # 添加定位参数
    parser.add_argument('urls_dir', type=str,
                        help="URL文件路径")
    parser.add_argument('-save_dir', type=str, default='./', 
                        help="保存的目录或文件名")
    args = parser.parse_args()
    
    target_urls_filepath = args.urls_dir
    target_save_dir = args.save_dir
    download_states = multiproc_download(target_urls_filepath, target_save_dir, max_workers=20)
    display_download_state(download_states, os.path.join(target_save_dir, 'downloaded_failed_urls.txt'))


